from tools import ctype, alhkp

class Morph:
  spv=['する', 'できる', 'える', 'いる', 'いう', 'なる', '出来る', 'しまう',
    'おこなう', '行う', 'およぶ', '用いる', 'したがう', 'ともなう', '伴う', 'もとづく']
  spn = ['さ', 'づくり', '向き', '作り']
  spp = ['す', 'ど', 'お', 'ご', '各', '御', '今', '同', '本', '約']
  spt = ['以下', '以外', '以後', '以降', '以上', '以前', '等', '他', '強', '以内',
    '以来', '間', '後', '時', '上', '下', '製', '前', '付近', '程度', '前後', '側',
    '中', '的', '内', '様', '用', '同士', '方', '当たり', '自身', '自体']
  ivt = ['上記', '以前', '以後', '近年', '今回', '次回', '前回', '従来', '最近',
    '今後', '今年', '昨年', '来年', 'その後', 'その他', 'この内', 'この前', 'この程',
    'この間', 'あり', 'つき', 'おき', 'おり', 'とり', '取り', 'とき', 'のみ',
    'もち', 'より']
  nmlf = ['動詞-自立', '形容詞-自立', '記号-アルファベット', '未知語']
  nmln = ['一般', '固有名詞', 'サ変接続', '形容動詞語幹']
  
  def __init__(self, ls=['', '', '', '']):
    self.name=ls[0]
    self.yomi=ls[1]
    self.bname=ls[2]
    self.hinshi=ls[3]
    if len(ls) >= 6:
      self.inftp=ls[4]
      self.katyo=ls[5]
    else:
      self.inftp=''
      self.katyo=''
    self.ms=self.hinshi.split('-')
  
  def cdmfp(self):
    if len(self.name)==1 and ctype(self.name)=='hk':
      return False
    elif  self.ms[0] == '名詞':
      if len(self.ms) > 1 and self.ms[1] == '代名詞':
        return False
      elif len(self.ms) > 2 and self.ms[2] == '助動詞語幹':
        return False
      elif alhkp(self.name):
        return False
      elif len(self.ms) > 1 and self.ms[1] == '接尾' and self.name in Morph.spn:
        return False
      elif self.name == '問題':
        return True
      elif self.ms[1] == 'ナイ形容詞語幹':
        return False
      else:
        return True
    elif self.ms[0] == '未知語':
      if ctype(self.name[0])=='ot' or ctype(self.name[len(self.name)-1])=='ot':
        return False
      else:
        return True
    elif self.hinshi == "記号-アルファベット":
      return True
    elif self.ms[0] == "接頭詞":
      if self.name in Morph.spp:
        return False
      else:
        return True
    elif self.ms[0] == '動詞':
      if self.katyo == '体言接続特殊２':
        return True
      elif self.katyo == '連用形':
        if  self.bname in Morph.spv:
          return False
        else:
          return True
    else:
      return False
  
  def nmlel(self):
    if self.hinshi in Morph.nmlf:
      return True
    if self.ms[0] == '名詞' and self.ms[1] in Morph.nmln:
      return True
    if self.ms[0] == '動詞' and self.katyo == '連用形':
      return True
    else:
      return False
  
  def lsncel(self):
    if len(self.ms) > 1 and self.ms[1] == '数':
      return True
    elif len(self.ms) > 2 and self.ms[2] == '助数詞':
      return True
    else:
      return False
  
  def lsspel(self):
    return self.lsncel() or self.name in Morph.spt
  
  def tpspel(self):
    return self.name in Morph.ivt

def mksent(str, chasen):
  ls=chasen.parse(str).rstrip().split("\n")
  rs=[]
  for el in ls:
    if el != 'EOS':
      rs.append(Morph(el.split("\t")))
  return rs

def vldemf(st, pt):
  vd = ['。', '　', '、', '．', '，', '／', '）', '（', '「', '」', '”', '：']
  if pt >= len(st):
    return True
  else:
    mf1 = st[pt]
  if mf1.ms[0] == '助詞' or mf1.ms[0] == '接続詞':
    return True
  elif mf1.ms[0] == '記号':
    if mf1.name in vd:
      return True
    else:
      return False
  elif mf1.ms[0] == '動詞':
    if mf1.bname == 'する' or mf1.bname =='できる':
      return True
    else:
      return False
  elif mf1.ms[0] == '助動詞' and st[pt - 1].ms[0] != '動詞':
    return True
  else:
    return False

def vldsmf(st, pt):
  if pt == 0:
    return True
  else:
    mf1 = st[pt - 1]
  if mf1.ms[0] == '名詞':
    return False
  elif mf1.ms[0] == '動詞' and mf1.katyo == '連用形':
    if mf1.bname != 'する' and mf1.bname != 'できる':
      return False
  elif mf1.name == "／" or mf1.name == "．":
    return False
  else:
    return True

def cndpss(st):
  ls = len(st)
  cd = []
  fp = -1
  sp = 0
  while sp < ls:
    if st[sp].cdmfp():
      if fp < 0:
        fp = sp
    else:
      if fp >= 0:
        if sp - fp > 1:
          cd.append([fp, sp])
      fp = -1
    sp = sp + 1
  if fp >= 0 and sp - fp > 1:
    cd.append([fp, sp])
  return cd

def tstedg(st, ps):
  if vldsmf(st, ps[0]) and vldemf(st, ps[1]):
    return True
  else:
    return False

def rmmgel(cd):
  wk = cd
  while (wk != [] and wk[0].tpspel()):
    wk = wk[1:len(wk)]
  while (wk != [] and wk[len(wk) - 1].lsspel()):
    wk = wk[0:len(wk) - 1]
  return wk

def vldtm(cd):
  if len(cd) < 2:
    return False
  elif cd[len(cd) - 1].ms[0] == '接頭詞':
    return False
  elif len(cd[0].ms) > 1 and cd[0].ms[1] == '接尾':
    return False
  elif len(cd[0].ms) > 2 and cd[0].ms[2] == '数':
    return False
  elif cd[0].hinshi == '名詞-サ変接続' and cd[1].hinshi == '動詞-自立':
    return False
  rt = False
  for el in cd:
    if el.nmlel():
      rt = True
  return rt

def tmext(sr, chasen):
  rs = []
  ws = mksent(sr, chasen)
  for el in cndpss(ws):
    if tstedg(ws, el):
      wk = rmmgel(ws[el[0]:el[1]])
      if vldtm(wk):
        rs.append(wk)
  return rs

def tmdata(ls):
  nm = ls[0].name
  ym = ls[0].yomi
  mf = ls[0].name
  hs = ls[0].hinshi
  for el in ls[1:len(ls)]:
    nm=nm+el.name
    ym=ym+el.yomi
    mf=mf+" %s" % el.name
    hs=hs+" // %s" % el.hinshi
  return [nm, ym, mf, hs]

